// rdparseb.h
// Glenn G. Chappell
// 11 Feb 2013
//
// For CS 331 Spring 2013
// Header for class RDParseb
// Second Recursive-Descent Parser

#ifndef FILE_RDPARSEB_H_INCLUDED
#define FILE_RDPARSEB_H_INCLUDED

#include "lex.h"   // For class Lex
#include <string>  // For std::string


// class RDParseb
// Simple Recursive-Descent parser
// Parses based on the grammar below.
// After the input string has been set using either a constructor or
//  member function set, return values have the following meanings:
//
//     parse() returns true; done() returns true
//         Correct parse of entire input string.
//
//     parse() returns true; done() returns false
//         Correct parse of beginning of string; additional characters
//         found past the end of the correctly parsed substring.
//
//     parse() returns false
//         Incorrect parse.
//
//  Grammar (start symbol is thing):
//
//      expr -> ID { ( "+" | "-" ) ID }
class RDParseb {
// ***** RDParseb: ctors, dctor, op= *****
public:

    // Ctor from string & default ctor
    // Sets input to given string. Sets parsed_ member.
    RDParseb(const std::string & input="")
    {
        set(input);
    }

    // Compiler-generated copy ctor, copy =, dctor used

// ***** RDParseb: general public functions *****
public:

    // set
    // Sets input to given string. Sets parsed_ member.
    void set(const std::string & input)
    {
        luthor_.set(input);
        parsed_ = parse_expr();
    }

    // parse
    // Returns true if the input parses correctly.
    bool parse()
    {
        return parsed_;
    }

    // done
    // Returns true if all lexemes in the input have been read.
    // If parse() returns true and done() returns false, then the input
    //  begins with a syntactically correct string, but there are
    //  additional characters after its end.
    bool done() const
    {
        return luthor_.done();
    }

// ***** RDParseb: internal-use functions *****
private:

    // matchString
    // Returns true if string part of the current lexeme is equal to the
    //  given string; otherwise returns false. When true is returned,
    //  the lexer is advanced.
    bool matchString(const std::string & str)
    {
        if (luthor_.current().first == str)
        {
            luthor_.advance();
            return true;
        }
        return false;
    }

    // matchToken
    // Returns true if token part of the current lexeme is equal to the
    //  given token; otherwise returns false. When true is returned, the
    //  lexer is advanced.
    bool matchToken(Lex::Token tok)
    {
        if (luthor_.current().second == tok)
        {
            luthor_.advance();
            return true;
        }
        return false;
    }

    // Parsing functions for nonterminals in the grammar
    // Each function parses the nonterminal in its name. A true return
    //  value means a correct parse, and the current lexeme is just past
    //  the end of the string the nonterminal expanded into. A false
    //  return means an invorrect parse; in this case the current lexeme
    //  is undefined.

    // parse_expr
    bool parse_expr();

// ***** RDParseb: data members *****
private:

    Lex luthor_;    // Our lexer
    bool parsed_;  // true if input parses correctly

};

#endif //#ifndef FILE_RDPARSEB_H_INCLUDED

